@@ -148,10 +148,16 @@ class WorkerPool:
             self._workers.append(self._start_worker(worker_id))
         logger.debug("Workers created")
 
-    def _restart_workers(self) -> None:
+    def _check_worker_status(self) -> List[Any]:
         """
-        Restarts workers that need to be restarted.
+        Checks the worker status:
+        - If the worker is supposed to be alive, but isn't, terminate.
+        - Restarts workers that need to be restarted.
+
+        :return: List of unordered results produces by workers
         """
+        # Check restarts
+        obtained_results = []
         for worker_id in self._worker_comms.get_worker_restarts():
             # Obtain results from exit results queue (should be done before joining the worker)
             if self.map_params.worker_exit:
@@ -159,11 +165,38 @@ class WorkerPool:
 
             # Join worker
             self._worker_comms.reset_worker_restart(worker_id)
-            self._workers[worker_id].join()
+            self._workers[worker_id].join(timeout=0.01)
+
+            # If we time-out, it means the worker still has data that needs to be send over. Note that on Windows, this
+            # is not necessarily the case. Windows is just a bit slow and thinks it cannot join yet, while in fact
+            # nothing is stoping it from joining. So, we just try it a few times ...
+            while self._workers[worker_id].exitcode is None:
+                try:
+                    obtained_results.append(self._worker_comms.get_results(block=True, timeout=0.01))
+                except queue.Empty:
+                    pass
+
+                self._workers[worker_id].join(timeout=0.01)
 
             # Start new worker
             self._workers[worker_id] = self._start_worker(worker_id)
 
+        # Check that workers that are supposed to be alive, are actually alive. If not, then a worker died unexpectedly.
+        # Note that a worker can be alive, but their alive status is still False. This doesn't really matter, because we
+        # know the worker is alive according to the OS. The only way we know that something bad happened is when a
+        # worker is supposed to be alive but according to the OS it's not.
+        for worker_id in range(self.pool_params.n_jobs):
+            with self._worker_comms.get_worker_dead_lock(worker_id):
+                worker_died = self._worker_comms.is_worker_alive(worker_id) and not self._workers[worker_id].is_alive()
+            if worker_died:
+                # We need to add an exception if we're using the progress bar handler
+                if self._worker_comms.has_progress_bar():
+                    self._worker_comms.add_exception(RuntimeError, f"Worker-{worker_id} died unexpectedly")
+                self.terminate()
+                raise RuntimeError(f"Worker-{worker_id} died unexpectedly")
+
+        return obtained_results
+
     def _start_worker(self, worker_id: int) -> mp.Process:
         """
         Creates and starts a single worker
@@ -531,8 +564,10 @@ class WorkerPool:
                         except queue.Empty:
                             pass
 
-                        # Restart workers if necessary
-                        self._restart_workers()
+                        # Check worker status (e.g., restarts). This can yield intermediate results
+                        for results in self._check_worker_status():
+                            yield from results
+                            n_active -= 1
 
                     # Obtain the results not yet obtained
                     while not self._worker_comms.exception_thrown() and n_active != 0:
@@ -542,8 +577,10 @@ class WorkerPool:
                         except queue.Empty:
                             pass
 
-                        # Restart workers if necessary
-                        self._restart_workers()
+                        # Check worker status (e.g., restarts). This can yield intermediate results
+                        for results in self._check_worker_status():
+                            yield from results
+                            n_active -= 1
 
                     # Terminate if exception has been thrown at this point
                     if self._worker_comms.exception_thrown():
@@ -648,7 +685,7 @@ class WorkerPool:
                 t.join(timeout=0.01)
                 if not t.is_alive():
                     break
-                self._restart_workers()
+                self._check_worker_status()
             logger.debug("Done joining task queues")
 
             # When an exception occurred in the above process (i.e., the worker init function raises), we need to handle
