@@ -1,9 +1,12 @@
 import logging
+import os
+import signal
 import types
 import unittest
 import warnings
 from itertools import product, repeat
 from multiprocessing import Barrier, Value
+from threading import Thread
 from unittest.mock import patch
 
 import numpy as np
@@ -24,6 +27,10 @@ def square(idx, x):
     return idx, x * x
 
 
+def extremely_large_output(idx, _):
+    return idx, os.urandom(1024 * 1024)
+
+
 def square_numpy(x):
     return x * x
 
@@ -55,7 +62,6 @@ class MapTest(unittest.TestCase):
             yield from iterable
 
         # Test results for different parameter settings
-        print()
         for n_jobs, n_tasks_max_active, worker_lifespan, chunk_size, n_splits in tqdm([
             (None, None, None, None, None),
             (1, None, None, None, None),
@@ -78,7 +84,8 @@ class MapTest(unittest.TestCase):
                                       chunk_size=chunk_size, n_splits=n_splits):
 
                         # Test if parallel map results in the same as ordinary map function. Should work both for
-                        # generators and iterators. Also check if an empty list works as desired.
+                        # generators and iterators. Also check if an empty list and extremely large output (exceeding
+                        # os.pipe limits) works as desired.
                         results_list = map_func(square, self.test_data, max_tasks_active=n_tasks_max_active,
                                                 worker_lifespan=worker_lifespan)
                         self.assertIsInstance(results_list, result_type)
@@ -104,6 +111,16 @@ class MapTest(unittest.TestCase):
                         self.assertIsInstance(results_list, result_type)
                         self.assertEqual([], list(results_list))
 
+                    # When the os pipe capacity is exceeded, a worker restart based on worker lifespan would hang if we
+                    # not fetch all the results from a worker. We only verify the amount of data returned here.
+                    with self.subTest(map_func=map_func, output='data exceeding pipe limits', n_jobs=n_jobs,
+                                      n_tasks_max_active=n_tasks_max_active, worker_lifespan=worker_lifespan,
+                                      chunk_size=chunk_size, n_splits=n_splits):
+                        results_list = map_func(extremely_large_output, self.test_data,
+                                                max_tasks_active=n_tasks_max_active, worker_lifespan=worker_lifespan)
+                        self.assertIsInstance(results_list, result_type)
+                        self.assertEqual(len(self.test_desired_output), len(list(results_list)))
+
     def test_numpy_input(self):
         """
         Test map with numpy input
@@ -676,9 +693,9 @@ class DaemonTest(unittest.TestCase):
             pool.map(self._square_daemon, ((X,) for X in repeat(self.test_data, 3)), chunk_size=1)
 
     @staticmethod
-    def _square_daemon(X):
+    def _square_daemon(x):
         with WorkerPool(n_jobs=4) as pool:
-            return pool.map(square, X, chunk_size=1)
+            return pool.map(square, x, chunk_size=1)
 
 
 class CPUPinningTest(unittest.TestCase):
@@ -707,7 +724,7 @@ class CPUPinningTest(unittest.TestCase):
                                                (4, [[0, 3]], [[0, 3], [0, 3], [0, 3], [0, 3]])]:
             # The test has been designed for a system with at least 4 cores. We'll skip those test cases where the CPU
             # IDs exceed the number of CPUs.
-            if cpu_ids is not None and np.array(cpu_ids).max() >= cpu_count():
+            if cpu_ids is not None and np.array(cpu_ids).max(initial=0) >= cpu_count():
                 continue
 
             with self.subTest(n_jobs=n_jobs, cpu_ids=cpu_ids), patch('mpire.pool.set_cpu_affinity') as p, \
@@ -1141,6 +1158,52 @@ class ExceptionTest(unittest.TestCase):
                 with self.subTest(function='square_raises_on_idx', map='imap'), self.assertRaises(ValueError):
                     list(pool.imap_unordered(self._square_raises_on_idx, self.test_data, progress_bar=progress_bar))
 
+    def test_defunct_processes_exit(self):
+        """
+        Tests if MPIRE correctly shuts down after process becomes defunct using exit()
+        """
+        print()
+        for n_jobs, progress_bar, worker_lifespan in [(1, False, None),
+                                                      (3, True, 1),
+                                                      (3, False, 3)]:
+            for start_method in TEST_START_METHODS:
+                # Progress bar on Windows + threading is not supported right now
+                if RUNNING_WINDOWS and start_method == 'threading' and progress_bar:
+                    continue
+                self.logger.debug(f"========== {start_method}, {n_jobs}, {progress_bar}, {worker_lifespan} ==========")
+                with self.subTest(n_jobs=n_jobs, progress_bar=progress_bar, worker_lifespan=worker_lifespan,
+                                  start_method=start_method), self.assertRaises(SystemExit), \
+                        WorkerPool(n_jobs=n_jobs, start_method=start_method) as pool:
+                    pool.map(self._exit, range(100), progress_bar=progress_bar, worker_lifespan=worker_lifespan)
+
+    def test_defunct_processes_kill(self):
+        """
+        Tests if MPIRE correctly shuts down after one process becomes defunct using os.kill().
+
+        We kill worker 0 and to be sure it's alive we set an event object and then go in an infinite loop. The kill
+        thread waits until the event is set and then kills the worker. The other workers are also ensured to have done
+        something so we can test what happens during restarts
+        """
+        print()
+        for n_jobs, progress_bar, worker_lifespan in [(1, False, None),
+                                                      (3, True, 1),
+                                                      (3, False, 3)]:
+            for start_method in TEST_START_METHODS:
+                # Can't kill threads
+                if start_method == 'threading':
+                    continue
+
+                self.logger.debug(f"========== {start_method}, {n_jobs}, {progress_bar}, {worker_lifespan} ==========")
+                with self.subTest(n_jobs=n_jobs, progress_bar=progress_bar, worker_lifespan=worker_lifespan,
+                                  start_method=start_method), self.assertRaises(RuntimeError), \
+                        WorkerPool(n_jobs=n_jobs, pass_worker_id=True, start_method=start_method) as pool:
+                    events = [pool.ctx.Event() for _ in range(n_jobs)]
+                    kill_thread = Thread(target=self._kill_process, args=(events[0], pool))
+                    kill_thread.start()
+                    pool.set_shared_objects(events)
+                    pool.map(self._worker_0_sleeps_others_square, range(1000), progress_bar=progress_bar,
+                             worker_lifespan=worker_lifespan, chunk_size=1)
+
     @staticmethod
     def _square_raises(_, x):
         raise ValueError(x)
@@ -1151,3 +1214,29 @@ class ExceptionTest(unittest.TestCase):
             raise ValueError(x)
         else:
             return idx, x * x
+
+    @staticmethod
+    def _exit(_):
+        exit()
+
+    @staticmethod
+    def _worker_0_sleeps_others_square(worker_id, events, x):
+        """
+        Worker 0 waits until the other workers have at least spun up and then sets her event and sleeps
+        """
+        if worker_id == 0:
+            [event.wait() for event in events[1:]]
+            events[0].set()
+            while True:
+                pass
+        else:
+            events[worker_id].set()
+            return x * x
+
+    @staticmethod
+    def _kill_process(event, pool):
+        """
+        Wait for event and kill
+        """
+        event.wait()
+        pool._workers[0].terminate()
